<?php

namespace Drupal\layout_builder_awesome_sections\Plugin\Layout;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefault;
use Drupal\Core\Plugin\PluginFormInterface;

/**
 * Class AwesomeLayout.
 *
 */
class AwesomeLayout extends LayoutDefault implements PluginFormInterface {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    /** @var \Drupal\Core\Layout\LayoutDefinition $plugin_definition */
    $plugin_definition = $this->getPluginDefinition();
    $regions = $plugin_definition->getRegions();
    $region_count = count($regions);
    $configuration['uuid'] = NULL;
    // Default column width configuration.
    if ($region_count > 1) {
      $column_width = [];
      foreach ($regions as $key => $region) {
        $column_width[$key] = round(1 / $region_count, 2) * 100;
      }
      if ($column_width == 3) {
        $column_width[array_keys($regions)[1]] = 34;
      }
      $configuration['section_properties']['column_width'] = $column_width;

      // Default mobile_breakpoint width configuration.
      $configuration['section_properties']['mobile_breakpoint'] = 576;
    }

    // Default section_class width configuration.
    $configuration['section_properties']['section_class'] = '';


    foreach ($regions as $key => $region) {
      $region_properties[$key] = [];
      $region_properties[$key]['background_color'] = '#ffffff';
      $region_properties[$key]['class'] = '';
      $region_properties[$key]['padding-left'] = 0;
      $region_properties[$key]['padding-right'] = 0;
      $region_properties[$key]['padding-top'] = 0;
      $region_properties[$key]['padding-bottom'] = 0;
    }
    $configuration = $configuration + $region_properties;

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Layout\LayoutDefinition $plugin_definition */
    $plugin_definition = $this->getPluginDefinition();
    $regions = $plugin_definition->getRegions();
    $region_count = count($regions);

    // Section properties
    $form['section_properties'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this
        ->t('Section properties'),
    ];

    // Section properties: column width
    if ($region_count > 1) {

      foreach ($regions as $key => $region) {
        $form['section_properties']['column_width'][$key] = [
          '#type' => 'number',
          '#min' => 1,
          '#max' => 99,
          'step' => 1,
          '#title' => $this
            ->t('Region ' . $region['label'] . ' width (%)'),
          '#default_value' => $this->configuration['section_properties']['column_width'][$key],
        ];
      }
    }

    // Section properties: mobile Breakpoint
    if ($region_count > 1) {
      $form['section_properties']['mobile_breakpoint'] = [
        '#type' => 'number',
        '#title' => $this
          ->t('Mobile breakpoint width (px)'),
        '#min' => 1,
        '#max' => 2160,
        'step' => 1,
        '#default_value' => $this->configuration['section_properties']['mobile_breakpoint'],
      ];
    }

    // Section properties: class
    $form['section_properties']['section_class'] = [
      '#type' => 'textfield',
      '#title' => $this
        ->t('Section class'),
      '#default_value' => $this->configuration['section_properties']['section_class'],
      '#size' => 60,
      '#maxlength' => 128,
    ];

    // Each region properties, closed by default.
    foreach ($regions as $key => $region) {
      $form[$key] = [
        '#type' => 'details',
        '#title' => $this
          ->t('Region ' . $region['label'] . ' properties'),
      ];

      if ($region_count > 1) {
        $form[$key]['background_color'] = [
          '#type' => 'color',
          '#title' => $this
            ->t('Background color'),
          '#default_value' => $this->configuration[$key]['background_color'],
        ];
      }

      $form[$key]['padding_left'] = [
        '#type' => 'number',
        '#title' => $this
          ->t('Padding left (px)'),
        '#min' => 0,
        '#max' => 100,
        'step' => 1,
        '#default_value' => $this->configuration[$key]['padding_left'],
      ];
      $form[$key]['padding_right'] = [
        '#type' => 'number',
        '#title' => $this
          ->t('Padding right (px)'),
        '#min' => 0,
        '#max' => 100,
        'step' => 1,
        '#default_value' => $this->configuration[$key]['padding_right'],
      ];
      $form[$key]['padding_top'] = [
        '#type' => 'number',
        '#title' => $this
          ->t('Padding Top (px)'),
        '#min' => 0,
        '#max' => 100,
        'step' => 1,
        '#default_value' => $this->configuration[$key]['padding_top'],
      ];
      $form[$key]['padding_bottom'] = [
        '#type' => 'number',
        '#title' => $this
          ->t('Padding bottom (px)'),
        '#min' => 0,
        '#max' => 100,
        'step' => 1,
        '#default_value' => $this->configuration[$key]['padding_bottom'],
      ];

      $form[$key]['region_class'] = [
        '#type' => 'textfield',
        '#title' => $this
          ->t('Class'),
        '#default_value' => $this->configuration[$key]['region_class'],
        '#size' => 60,
        '#maxlength' => 128,
      ];
    }

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    /** @var \Drupal\Core\Layout\LayoutDefinition $plugin_definition */
    $plugin_definition = $this->getPluginDefinition();
    $regions = $plugin_definition->getRegions();
    if (!isset($this->configuration['uuid'])) {
      $this->configuration['uuid'] = \Drupal::service('uuid')->generate();
    }
    $this->configuration['section_properties'] = $form_state->getValue('section_properties');
    $this->configuration['section_class'] = $form_state->getValue('section_class');

    foreach ($regions as $key => $region) {
      $this->configuration[$key] = $form_state->getValue($key);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    $plugin_definition = $this->getPluginDefinition();
    // Notice $definition_regions is different with $regions.
    $definition_regions = $plugin_definition->getRegions();

    $build['#attributes']['class'] = [
      'layout',
      $this->getPluginDefinition()->getTemplate(),
      'uuid-'.$this->configuration['uuid'],
      $this->configuration['section_properties']['section_class'],
    ];
    if (count($definition_regions) > 1) {
      $build['#attributes']['style'] = ["display:flex;"];
    }
    $mobile_breakpoint = $this->configuration['section_properties']['mobile_breakpoint'] ?? NULL;
    if (!empty($mobile_breakpoint)) {
      $build['#attributes']['class'][] = 'mobile-breakpoint-' . $mobile_breakpoint;
    }
    foreach ($definition_regions as $key => $region) {
      if (isset($this->configuration[$key]['region_class'])) {
        $build[$key]['#attributes']['class'] = [$this->configuration[$key]['region_class']];
      }
      if (isset($this->configuration[$key]['background_color'])) {
        $build[$key]['#attributes']['style'][] = "background-color:" . $this->configuration[$key]['background_color'] . ";";
      }
      if (isset($this->configuration['section_properties']['column_width'][$key])) {
        $build[$key]['#attributes']['style'][] = "width:" . $this->configuration['section_properties']['column_width'][$key] . '%;';
      }
      if (isset($this->configuration[$key]['padding_left'])) {
        $build[$key]['#attributes']['style'][] = "padding-left:" . $this->configuration[$key]['padding_left'] . "px;";
      }
      if (isset($this->configuration[$key]['padding_top'])) {
        $build[$key]['#attributes']['style'][] = "padding-top:" . $this->configuration[$key]['padding_top'] . "px;";
      }
      if (isset($this->configuration[$key]['padding_right'])) {
        $build[$key]['#attributes']['style'][] = "padding-right:" . $this->configuration[$key]['padding_right'] . "px;";
      }
      if (isset($this->configuration[$key]['padding_bottom'])) {
        $build[$key]['#attributes']['style'][] = "padding-bottom:" . $this->configuration[$key]['padding_bottom'] . "px;";
      }
    }
    return $build;
  }

}
